// EOControlAssociation
// Enterprise Objects Framework
// Copyright (c) 1995, NeXT Software, Inc.  All rights reserved. 
//
// The EOControlAssociation & EOActionCellAssociation bind a control
// or cell to properties of the currently selected EO in a DisplayGroup.
// Supported Bindings:
//     value:     property of the EO to be displayed in the column.
//     enabled:   BOOL property of EO indicating whether control should be enabled
// Because setObjectValue:, and objectValue are used to communicate with
// the control, these associations work with NSImageView and NSImageCell.

#import <EOInterface/EOAssociation.h>

// Generic superclass handling Cells and Controls
@interface EOGenericControlAssociation : EOAssociation
{
    // our object can be either a NSControl subclass or an NSActionCell
    BOOL _didChange;  // has the text changed since the last AssocDidEdit?
    id _lastValue;
}
// must be overriden by subclasses
- (NSControl *)control;
    // get the control (we can be its delegate)
- (EOGenericControlAssociation *)editingAssociation;
    // What association is responsible for textDelegate messages.
    // In the case of matrixes and TableViews, this is the association
    // for the editing cell
@end

// For views
@interface EOControlAssociation : EOGenericControlAssociation
{}
@end

// For cells
@interface EOActionCellAssociation : EOGenericControlAssociation
{}
@end

@interface NSImage (EOImageFactory)
+ imageWithData:(NSData *)imageData;
    // Equivalent to [[[NSImage alloc] initWithData:imageData] autorelease].
    // Creates an NSImage from tiff, bmp, ico, dib, and eps data.
    // Useful in EOAttribute custom datatype converion.
    // Use TIFFRepresentation to extract a TIFF from the NSImage.
@end
